#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2019-10-28
#
# Monitor status of Cisco flash file systems
#
# 2019-10-28: initial release
# 2019-11-04: added wato for cisco_flash
# 2021-07-31: rewritten for CMK 2.0
# 2024-04-07: some code cleanup
#

# snmpwalk sample
#
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.2.1.1 = INTEGER: 1
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.2.2.1 = INTEGER: 1
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.3.1.1 = INTEGER: 1
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.3.2.1 = INTEGER: 1
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.4.1.1 = Gauge32: 2908606464
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.4.2.1 = Gauge32: 4294967295
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.5.1.1 = Gauge32: 1756741632
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.5.2.1 = Gauge32: 4294967295
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.6.1.1 = Gauge32: 624
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.6.2.1 = Gauge32: 0
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.7.1.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.7.2.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.8.1.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.8.2.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.9.1.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.9.2.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.10.1.1 = STRING: "bootflash"
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.10.2.1 = STRING: "usb0"
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.11.1.1 = INTEGER: 2
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.11.2.1 = INTEGER: 2
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.12.1.1 = INTEGER: 63
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.12.2.1 = INTEGER: 63
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.13.1.1 = Counter64: 2908606464
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.13.2.1 = Counter64: 16424566784
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.14.1.1 = Counter64: 1756741632
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.14.2.1 = Counter64: 16424558592
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.15.1.1 = INTEGER: 10
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.15.2.1 = INTEGER: 10
#
# sample info
#
# [
#  [u'1.1', u'2908606464', u'1758527488', u'446', u'3', u'bootflash', u'2', u'2908606464', u'1758527488', u'10'],
#  [u'2.1', u'4294967295', u'4294967295', u'0', u'3', u'usb0', u'2', u'16423845888', u'7506280448', u'10']
# ]
#
#

from dataclasses import dataclass
from typing import Dict
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    Metric,
    OIDEnd,
    Result,
    SNMPTree,
    Service,
    State,
    all_of,
    check_levels,
    contains,
    exists,
    register,
    render,
)


@dataclass(frozen=True)
class Partition:
    size: int | None
    free_space: int | None
    used_space: int | None
    index: int | None
    flash_index: int | None
    need_erasure: bool | None
    low_space_notify_threshold: int | None
    file_count: int | None


_part_need_erasure = {
    '1': True,
    '2': False,
}


###########################################################################
#
#  DATA Parser function
#
###########################################################################


def parse_cisco_flash(string_table: StringTable) -> Dict[str, Partition]:
    partitions: Dict[str, Partition] = {}

    for partition in string_table:
        try:
            part_oid_end, part_size, part_free_pace, part_file_count, part_name, part_need_erasure, \
                part_size_extended, part_free_space_extended, part_low_space_notif_threshold = partition
        except ValueError:
            continue

        try:
            dev_index, part_index = part_oid_end.split('.')
        except ValueError:
            continue

        part_free_pace = int(part_free_pace) if part_free_pace.isdigit() else None
        part_free_space_extended = int(part_free_space_extended) if part_free_space_extended.isdigit() else None
        part_size = int(part_size) if part_size.isdigit() else None
        part_size_extended = int(part_size_extended) if part_size_extended.isdigit() else None

        free_space = part_free_space_extended if part_free_space_extended else part_free_pace
        size = part_size_extended if part_size_extended else part_size

        if size > 0:
            partitions.update({
                part_name: Partition(
                    index=int(part_index) if part_index.isdigit() else None,
                    flash_index=int(dev_index) if dev_index.isdigit() else None,
                    size=size,
                    free_space=free_space,
                    used_space=size - free_space,
                    file_count=int(part_file_count) if part_file_count.isdigit() else None,
                    need_erasure=_part_need_erasure.get(part_need_erasure, None),
                    low_space_notify_threshold=int(
                        part_low_space_notif_threshold
                    ) if part_low_space_notif_threshold.isdigit() else None
                )})

    return partitions


###########################################################################
#
#  INVENTORY function
#
###########################################################################


def discovery_cisco_flash(section: Dict[str, Partition]) -> DiscoveryResult:
    for key in section:
        yield Service(item=key)


###########################################################################
#
#  CHECK function
#
###########################################################################


def check_cisco_flash(item, params, section: Dict[str, Partition]) -> CheckResult:
    try:
        partition = section[item]
    except KeyError:
        return

    if partition.low_space_notify_threshold:
        space_warn = partition.size - ((partition.size / 100) * partition.low_space_notify_threshold)
    else:
        space_warn = partition.size

    yield from check_levels(
        value=partition.used_space,
        label='Space used',
        metric_name='cisco_flash_partusedspace',
        render_func=render.disksize,
        levels_upper=(space_warn, partition.size - 10),
        boundaries=(0, partition.size),
    )

    percent_used = 100 * partition.used_space / partition.size

    yield from check_levels(
        value=percent_used,
        label='Percent',
        levels_upper=params['levels_upper_percent'],
        render_func=render.percent,
        boundaries=(0, 100),
        metric_name='cisco_flash_percent_used'
    )

    if partition.need_erasure:
        yield Result(state=State.WARN, notice='Partition needs erasure')

    yield Metric(
        value=partition.file_count,
        name='cisco_flash_partfilecount',
        boundaries=(0, None)
    )

    for key, value in [
        ('Device index', partition.flash_index),
        ('Partition index', partition.index),
        ('Partition need erasure', partition.need_erasure),
        ('Partition low space notify threshold', partition.low_space_notify_threshold),
    ]:
        if value:
            yield Result(state=State.OK, notice=f'{key}: {value}')

    size = max(partition.size, 1)  # for size = 0 --> div/0
    size = size / 1000 / 1000
    used_space = partition.used_space / 1000 / 1000
    free_space = partition.free_space / 1000 / 1000

    yield Result(state=State.OK, notice=f'Used: {used_space:.2f} of {size:.2f} MB, {free_space:.2f} MB free')

    if partition.file_count:
        yield Result(state=State.OK, summary=f'File count: {partition.file_count}')


###########################################################################
#
#  SNMP section
#
###########################################################################


register.snmp_section(
    name='cisco_flash',
    parse_function=parse_cisco_flash,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.9.9.10.1.1.4.1.1',  #
        oids=[
            OIDEnd(),
            '4',  # ciscoFlashPartitionSize
            '5',  # ciscoFlashPartitionFreeSpace
            '6',  # ciscoFlashPartitionFileCount
            '10',  # ciscoFlashPartitionName
            '11',  # ciscoFlashPartitionNeedErasure
            '13',  # ciscoFlashPartitionSizeExtended
            '14',  # ciscoFlashPartitionFreeSpaceExtended
            '15',  # ciscoFlashPartitionLowSpaceNotifThreshold
        ]
    ),
    detect=all_of(
        contains('.1.3.6.1.2.1.1.1.0', 'cisco'),  # sysDescr
        exists('.1.3.6.1.4.1.9.9.10.1.1.4.1.1.10.*'),  # CISCO-FLASH-MIB::ciscoFlashDevicesSupported
    ))

register.check_plugin(
    name='cisco_flash',
    service_name='Flash partition %s',
    discovery_function=discovery_cisco_flash,
    check_function=check_cisco_flash,
    check_default_parameters={
        'levels_upper_percent': (85, 90),
    },
    check_ruleset_name='cisco_flash',
)
