#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2019-10-22 
#
# invetory of cisco flash devices
#
# 2019-10-22: initial release
# 2021-07-31: rewritten for CMK 2.0
# 2023-04-07: some code cleanup
#             moved the Flash inventory from 'hardware.components.flash' to 'hardware.flash'
#

from dataclasses import dataclass
from typing import List, Dict, Any

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    InventoryResult,
    StringTable,
)
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    OIDEnd,
)
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    SNMPTree,
    TableRow,
    all_of,
    contains,
    exists,
    register,
)


@dataclass
class Section:
    flash_devices: List[Dict[str, Any]]
    flash_chips: List[Dict[str, Any]]
    flash_partitions: List[Dict[str, Any]]


_dev_programming_jumper = {
    '1': 'installed',
    '2': 'not installed',
    '3': 'unknown',
}

_dev_removable = {
    '1': True,
    '2': False,
}

_PartStatus = {
    '1': 'Read Only',
    '2': 'run from flash',
    '3': 'Read Write',
}

_part_checksum_algorithm = {
    '1': 'simple Checksum',
    '2': 'undefined',
    '3': 'simple CRC',
}

_part_upgrade_method = {
    '1': 'unknown',
    '2': 'rxboot Flash Load Helper',
    '3': 'direct',
}

_part_need_erasure = {
    '1': True,
    '2': False,
}


def parse_inv_cisco_flash(string_table: List[StringTable]) -> Section | None:
    try:
        devices, chips, partitions = string_table
    except ValueError:
        return

    flash_devices = []
    for cflDevice in devices:
        try:
            oid_end, dev_size, dev_min_partition_size, dev_max_partitions, dev_partitions, dev_chip_count, \
                dev_name, dev_descr, dev_controller, dev_programming_jumper, dev_init_time, dev_removable, \
                dev_phy_ent_index, dev_name_extended, dev_size_extended, dev_min_partition_size_extended = cflDevice
        except ValueError:
            continue

        dev_size = int(dev_size) if dev_size.isdigit() else None
        dev_size_extended = int(dev_size_extended) if dev_size_extended.isdigit() else None
        dev_min_partition_size = int(dev_min_partition_size) if dev_min_partition_size.isdigit() else None
        dev_min_partition_size_extended = int(
            dev_min_partition_size_extended) if dev_min_partition_size_extended.isdigit() else None

        # change sizes to MB
        size = dev_size_extended if dev_size_extended else dev_size
        size = size / 1024 / 1024 if size else None

        min_partition_size = dev_min_partition_size_extended if dev_min_partition_size_extended else dev_min_partition_size
        min_partition_size = min_partition_size / 1024 / 1024 if min_partition_size else None

        flash_device = {}
        for key, value in [
            ('index', oid_end),
            ('name', dev_name),
            ('description', dev_descr),
            ('controller', dev_controller),
            ('programmingjumper', _dev_programming_jumper.get(dev_programming_jumper)),
            ('inittime', dev_init_time),
            ('removable', _dev_removable.get(dev_removable)),
            ('nameextended', dev_name_extended),
            ('physentindex', dev_phy_ent_index),
            ('size', size),
            ('minpartitionsize', min_partition_size),
            ('maxprtitions', dev_max_partitions),
            ('partitions', dev_partitions),
            ('chipcount', dev_chip_count),
        ]:
            if value:
                flash_device.update({key: value})
        flash_devices.append(flash_device)

    flash_chips = []

    for chip in chips:
        try:
            oid_end, chip_code, chip_descr, chip_write_retries, chip_erase_retries, chip_max_write_retries, \
                chip_max_erase_retries = chip
        except ValueError:
            continue

        try:
            dev_index, chip_index = oid_end.split('.')
        except ValueError:
            continue

        flash_chip = {}
        for key, value in [
            ('index', chip_index),
            ('flashindex', dev_index),
            ('code', chip_code),
            ('description', chip_descr),
            ('writeretries', chip_write_retries),
            ('eraseretries', chip_erase_retries),
            ('maxwriteretries', chip_max_write_retries),
            ('maxeraseretries', chip_max_erase_retries),
        ]:
            if value:
                flash_chip.update({key: value})

        flash_chips.append(flash_chip)

    flash_partitions = []
    for partition in partitions:
        try:
            part_oid_end, part_start_chip, part_end_chip, part_size, part_free_space, part_file_count, \
                part_checksum_algorithm, part_status, part_upgrade_method, part_name, part_need_erasure, \
                part_file_name_length, part_size_extended, part_free_space_extended, \
                part_low_space_notif_threshold = partition
        except ValueError:
            continue

        try:
            dev_index, part_index = part_oid_end.split('.')
        except ValueError:
            continue

        part_size = int(part_size) if part_size.isdigit() else None
        part_free_space = int(part_free_space) if part_free_space.isdigit() else None
        part_size_extended = int(part_size_extended) if part_size_extended.isdigit() else None
        part_free_space_extended = int(part_free_space_extended) if part_free_space_extended.isdigit() else None

        # change sizes to MB
        size = part_size_extended if part_size_extended else part_size
        size = size / 1024 / 1024 if size else None

        free_space = part_free_space_extended if part_free_space_extended else part_free_space
        free_space = free_space / 1024 / 1024 if free_space else None

        flash_partition = {}
        for key, value in [
            ('index', part_index),
            ('flashindex', dev_index),
            ('startchip', part_start_chip),
            ('endchip', part_end_chip),
            ('filecount', part_file_count),
            ('crcsumalgo', _part_checksum_algorithm.get(part_checksum_algorithm)),
            ('status', _PartStatus.get(part_status)),
            ('upgrademethod', _part_upgrade_method.get(part_upgrade_method)),
            ('name', part_name),
            ('neederasure', _part_need_erasure.get(part_need_erasure)),
            ('filenamelength', part_file_name_length),
            ('lowspacenotifythreshold', part_low_space_notif_threshold),
            ('size', size),
            ('freespace', free_space),
        ]:
            if value:
                flash_partition.update({key: value})

        flash_partitions.append(flash_partition)

    return Section(
        flash_devices=flash_devices,
        flash_chips=flash_chips,
        flash_partitions=flash_partitions
    )


def inventory_cisco_flash(params, section: Section) -> InventoryResult:
    flash_devices = section.flash_devices
    flash_chips = section.flash_chips
    flash_partitions = section.flash_partitions
    base_path = ['hardware', 'flash']
    # inventory of cisco flash devices
    remove_columns = []
    disabled = False
    # 
    if params:
        disabled = params.get('cflDeviceDisable', disabled)
        # get list of columns to remove from inventory
        remove_columns = params.get('cflDeviceRemovecolumns', remove_columns)

    if not disabled:
        path = base_path + ['devices']

        for flash_device in flash_devices:
            key_columns = {'index': flash_device['index'], }
            flash_device.pop('index')
            for key in flash_device.keys():
                if key in remove_columns:
                    flash_device.pop(key)

            yield TableRow(
                path=path,
                key_columns=key_columns,
                inventory_columns=flash_device
            )

    # inventory of cisco flash chips

    remove_columns = []
    disabled = False

    if params:
        disabled = params.get('cflChipDisable', disabled)
        # get list of columns to remove from inventory
        remove_columns = params.get('cflChipRemovecolumns', remove_columns)

    if not disabled:
        path = base_path + ['chips']

        for flash_chip in flash_chips:
            key_columns = {'index': flash_chip['index'], 'flashindex': flash_chip['flashindex']}
            flash_chip.pop('index')
            flash_chip.pop('flashindex')
            for key in flash_chip.keys():
                if key in remove_columns:
                    flash_chip.pop(key)

            yield TableRow(
                path=path,
                key_columns=key_columns,
                inventory_columns=flash_chip
            )

    # inventory of cisco flash partitions

    remove_columns = []
    disabled = False

    if params:
        disabled = params.get('cflPartitionDisable', disabled)
        # get list of columns to remove from inventory
        remove_columns = params.get('cflPartitionRemovecolumns', remove_columns)

    if not disabled:
        path = base_path + ['partitions']
        for flash_partition in flash_partitions:
            key_columns = {'index': flash_partition['index'], 'flashindex': flash_partition['flashindex']}
            flash_partition.pop('index')
            flash_partition.pop('flashindex')
            for key in flash_partition.keys():
                if key in remove_columns:
                    flash_partition.pop(key)

            yield TableRow(
                path=path,
                key_columns=key_columns,
                inventory_columns=flash_partition
            )


register.snmp_section(
    name='inv_cisco_flash',
    parse_function=parse_inv_cisco_flash,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.9.9.10.1.1.2.1',  #
            oids=[
                OIDEnd(),  # ifIndex
                '2',  # ciscoFlashDeviceSize
                '3',  # ciscoFlashDeviceMinPartitionSize
                '4',  # ciscoFlashDeviceMaxPartitions
                '5',  # ciscoFlashDevicePartitions
                '6',  # ciscoFlashDeviceChipCount
                '7',  # ciscoFlashDeviceName
                '8',  # ciscoFlashDeviceDescr
                '9',  # ciscoFlashDeviceController
                '11',  # ciscoFlashDeviceProgrammingJumper
                '12',  # ciscoFlashDeviceInitTime
                '13',  # ciscoFlashDeviceRemovable
                '14',  # ciscoFlashPhyEntIndex
                '15',  # ciscoFlashDeviceNameExtended
                '16',  # ciscoFlashDeviceSizeExtended
                '17',  # ciscoFlashDeviceMinPartitionSizeExtended
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.9.9.10.1.1.3.1.1',  #
            oids=[
                OIDEnd(),  # ifIndex
                '2',  # ciscoFlashChipCode
                '3',  # ciscoFlashChipDescr
                '4',  # ciscoFlashChipWriteRetries
                '5',  # ciscoFlashChipEraseRetries
                '6',  # ciscoFlashChipMaxWriteRetries
                '7',  # ciscoFlashChipMaxEraseRetries
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.9.9.10.1.1.4.1.1',  #
            oids=[
                OIDEnd(),  # ifIndex
                '2',  # ciscoFlashPartitionStartChip
                '3',  # ciscoFlashPartitionEndChip
                '4',  # ciscoFlashPartitionSize
                '5',  # ciscoFlashPartitionFreeSpace
                '6',  # ciscoFlashPartitionFileCount
                '7',  # ciscoFlashPartitionChecksumAlgorithm
                '8',  # ciscoFlashPartitionStatus
                '9',  # ciscoFlashPartitionUpgradeMethod
                '10',  # ciscoFlashPartitionName
                '11',  # ciscoFlashPartitionNeedErasure
                '12',  # ciscoFlashPartitionFileNameLength
                '13',  # ciscoFlashPartitionSizeExtended
                '14',  # ciscoFlashPartitionFreeSpaceExtended
                '15',  # ciscoFlashPartitionLowSpaceNotifThreshold
            ]
        ),
    ],
    detect=all_of(
        contains('.1.3.6.1.2.1.1.1.0', 'cisco'),
        exists('.1.3.6.1.4.1.9.9.10.1.1.1.*'),  # CISCO-FLASH-MIB::ciscoFlashDevicesSupported
    )
)

register.inventory_plugin(
    name='inv_cisco_flash',
    inventory_function=inventory_cisco_flash,
    inventory_default_parameters={},
    inventory_ruleset_name='inv_cisco_flash',
)
